local DX = application:getLogicalTranslateX() / application:getLogicalScaleX()
local DY = application:getLogicalTranslateY() / application:getLogicalScaleY()
local W = application:getContentWidth()
local H = application:getContentHeight()

Screen = {
	W = W,
	H = H,
	FW = DX*2 + W,
	FH = DY*2 + H,
	Left = -DX,
	Right = -DX + W + DX*2,
	Top = -DY,
	Bottom = -DY + H + DY*2,
	Center = {x = W/2, y = H/2}
}

require "box2d"

local world = b2.World.new(0, 10)
local mainBox = Box.new(100, 100)
mainBox:setPosition(Screen.Center.x, Screen.Center.y)
mainBox:createBody(world, b2.DYNAMIC_BODY, "mainBox", 0.5, 0.3, 0.25)
mainBox:getBody():setAngle(math.random(360))
mainBox:reColor({1,0,0})
stage:addChild(mainBox)
local boxes = {}
local count_x = 5
local count_y = 3
local width = Screen.FW / count_x
local height = Screen.FH / count_y

function lerp(a,b,t) return (1-t)*a + t*b end
function createBounds()
	-- leftBound
	local bound = Box.new(10, Screen.FH)
	bound:createBody(world, b2.STATIC_BODY, "bound")
	bound:setPosition2(Screen.Left + 5, Screen.Center.y)
	bound:reColor({0,0,0})
	stage:addChild(bound)
	-- rightBound
	bound = Box.new(10, Screen.FH)
	bound:createBody(world, b2.STATIC_BODY, "bound")
	bound:setPosition2(Screen.Right - 5, Screen.Center.y)
	bound:reColor({0,0,0})
	stage:addChild(bound)
	-- topBound
	bound = Box.new(Screen.FW, 10)
	bound:createBody(world, b2.STATIC_BODY, "bound")
	bound:setPosition2(Screen.Center.x, Screen.Top + 5)
	bound:reColor({0,0,0})
	stage:addChild(bound)
end

function AppStarts(e)
	local k = 1.1
	local d, f, r = 0,1,0
	for i = 1, count_x do
		local b = Box.new(width, Screen.FH)
		b:reColor()
		b:setPosition(Screen.Left + width/2 + (i - 1) * width, Screen.Center.y + Screen.FH / k)
		b:createBody(world, b2.STATIC_BODY, "wall", d, f, r)
		stage:addChild(b)
		table.insert(boxes, {box = b, dx = 0, dy = 0, id = -1, touched = false, dir = "v"})
	end
	for i = 1, count_y do
		local b = Box.new(Screen.FW, height)
		b:reColor()
		b:setPosition(Screen.Center.x - Screen.FW / k, Screen.Top + height/2 + (i - 1) * height)
		b:createBody(world, b2.STATIC_BODY, "wall", d, f, r)
		stage:addChild(b)
		table.insert(boxes, {box = b, dx = 0, dy = 0, id = -1, touched = false, dir = "h"})
		
		b = Box.new(Screen.FW, height)
		b:reColor()
		b:setPosition(Screen.Center.x + Screen.FW / k, Screen.Top + height/2 + (i - 1) * height)
		b:createBody(world, b2.STATIC_BODY, "wall", d, f, r)
		stage:addChild(b)
		table.insert(boxes, {box = b, dx = 0, dy = 0, id = -1, touched = false, dir = "h"})
	end
	createBounds()
	
	local Draw = b2.DebugDraw.new()
	world:setDebugDraw(Draw)
	--stage:addChild(Draw)
end
--
local dx, dy = 0, 0
function updateMainBox()
	local b = mainBox:getBody()
	local x, y = b:getPosition()
	local ang = b:getAngle()
	--[[
	-- keep mainBox in the center
	dx, dy = b:getLinearVelocity()
	
	if (x < Screen.Center.x) then 
		dx = lerp(dx, 5, 0.05)
	else
		dx = lerp(dx, -5, 0.05)
	end
	b:setLinearVelocity(dx, dy)
	]]
	mainBox:setPosition(x, y)
	mainBox:setRotation(ang * 180 / math.pi)
end
--
function EnterFrame(e)
	world:step(1/60, 8, 8)
	updateMainBox()
end
--
function TouchBegin(e)
	local x, y = e.touch.x, e.touch.y
	for i = 1, #boxes do
		local el = boxes[i]
		if (el.box:hitTestPoint(x, y)) then
			el.dx = x - el.box:getX()
			el.dy = y - el.box:getY()
			el.touched = true
			el.id = e.touch.id
			e:stopPropagation()
			break
		end
	end
end
--
function TouchMove(e)
	local x, y = e.touch.x, e.touch.y
	for i = 1, #boxes do
		local el = boxes[i]
		if (el.id == e.touch.id and el.touched) then
			-- vertical drag
			if (el.dir == "v") then 
				el.box:setY2(y - el.dy)
			-- horizontal drag
			else
				el.box:setX2(x - el.dx)
			end
		end
	end
end
--
function TouchEnd(e)
	local x, y = e.touch.x, e.touch.y
	for i = 1, #boxes do
		local el = boxes[i]
		if (el.id == e.touch.id and el.touched) then
			el.touched = false
			el.id = -1
			el.dx = -1
			el.dy = -1
		end
	end
end
--

stage:addEventListener(Event.APPLICATION_START, AppStarts)
stage:addEventListener(Event.TOUCHES_BEGIN, TouchBegin)
stage:addEventListener(Event.ENTER_FRAME, EnterFrame)
stage:addEventListener(Event.TOUCHES_MOVE, TouchMove)
stage:addEventListener(Event.TOUCHES_END, TouchEnd)
